@@ -1,24 +1,29 @@
-"""
-    weasyprint.images
-    -----------------
-
-    Fetch and decode images in various formats.
-
-"""
+"""Fetch and decode images in various formats."""
 
+import io
 import math
+import struct
+from hashlib import md5
 from io import BytesIO
 from itertools import cycle
+from math import inf
+from pathlib import Path
+from urllib.parse import urlparse
+from urllib.request import url2pathname
 from xml.etree import ElementTree
 
 import pydyf
-from PIL import Image
+from PIL import Image, ImageFile, ImageOps
 
+from . import DEFAULT_OPTIONS
 from .layout.percent import percentage
 from .logger import LOGGER
 from .svg import SVG
 from .urls import URLFetchingError, fetch
 
+# Don’t crash when converting truncated images
+ImageFile.LOAD_TRUNCATED_IMAGES = True
+
 
 class ImageLoadingError(ValueError):
     """An error occured when loading an image.
@@ -35,32 +40,200 @@ class ImageLoadingError(ValueError):
 
 
 class RasterImage:
-    def __init__(self, pillow_image, image_id, optimize_size):
-        pillow_image.id = image_id
-        self._pillow_image = pillow_image
-        self._optimize_size = optimize_size
-        self._intrinsic_width = pillow_image.width
-        self._intrinsic_height = pillow_image.height
-        self._intrinsic_ratio = (
-            self._intrinsic_width / self._intrinsic_height
-            if self._intrinsic_height != 0 else float('inf'))
+    def __init__(self, pillow_image, image_id, image_data, filename=None,
+                 cache=None, orientation='none', options=DEFAULT_OPTIONS):
+        # Transpose image
+        original_pillow_image = pillow_image
+        pillow_image = rotate_pillow_image(pillow_image, orientation)
+        if original_pillow_image is not pillow_image:
+            # Keep image format as it is discarded by transposition
+            pillow_image.format = original_pillow_image.format
+            # Discard original data, as the image has been transformed
+            image_data = filename = None
 
-    def get_intrinsic_size(self, image_resolution, font_size):
-        return (
-            self._intrinsic_width / image_resolution,
-            self._intrinsic_height / image_resolution,
-            self._intrinsic_ratio)
+        self.id = image_id
+        self._cache = {} if cache is None else cache
+        self._jpeg_quality = jpeg_quality = options['jpeg_quality']
+        self._dpi = options['dpi']
+
+        if 'transparency' in pillow_image.info:
+            pillow_image = pillow_image.convert('RGBA')
+        elif pillow_image.mode in ('1', 'P', 'I'):
+            pillow_image = pillow_image.convert('RGB')
+
+        self.mode = pillow_image.mode
+        self.width = pillow_image.width
+        self.height = pillow_image.height
+        self.ratio = (self.width / self.height) if self.height != 0 else inf
+        self.optimize = optimize = options['optimize_images']
+
+        if pillow_image.format in ('JPEG', 'MPO'):
+            self.format = 'JPEG'
+            if image_data is None or optimize or jpeg_quality is not None:
+                image_file = io.BytesIO()
+                options = {'format': 'JPEG', 'optimize': optimize}
+                if self._jpeg_quality is not None:
+                    options['quality'] = self._jpeg_quality
+                pillow_image.save(image_file, **options)
+                image_data = image_file.getvalue()
+                filename = None
+        else:
+            self.format = 'PNG'
+            if image_data is None or optimize or pillow_image.format != 'PNG':
+                image_file = io.BytesIO()
+                pillow_image.save(image_file, format='PNG', optimize=optimize)
+                image_data = image_file.getvalue()
+                filename = None
+        self.image_data = self.cache_image_data(image_data, filename)
+
+    def get_intrinsic_size(self, resolution, font_size):
+        return self.width / resolution, self.height / resolution, self.ratio
 
     def draw(self, stream, concrete_width, concrete_height, image_rendering):
-        if self._intrinsic_width <= 0 or self._intrinsic_height <= 0:
+        if self.width <= 0 or self.height <= 0:
             return
 
-        image_name = stream.add_image(
-            self._pillow_image, image_rendering, self._optimize_size)
+        width, height = self.width, self.height
+        interpolate = 'true' if image_rendering == 'auto' else 'false'
+        ratio = 1
+        if self._dpi:
+            pt_to_in = 4 / 3 / 96
+            width_inches = abs(concrete_width * stream.ctm[0][0] * pt_to_in)
+            height_inches = abs(concrete_height * stream.ctm[1][1] * pt_to_in)
+            dpi = max(self.width / width_inches, self.height / height_inches)
+            if dpi > self._dpi:
+                ratio = self._dpi / dpi
+        image_name = stream.add_image(self, width, height, interpolate, ratio)
+
         stream.transform(
             concrete_width, 0, 0, -concrete_height, 0, concrete_height)
         stream.draw_x_object(image_name)
 
+    def cache_image_data(self, data, filename=None, alpha=False):
+        if filename:
+            return LazyLocalImage(filename)
+        else:
+            key = f'{self.id}{int(alpha)}{self._dpi or ""}'
+            return LazyImage(self._cache, key, data)
+
+    def get_xobject(self, width, height, interpolate):
+        if self.mode in ('RGB', 'RGBA'):
+            color_space = '/DeviceRGB'
+        elif self.mode in ('L', 'LA'):
+            color_space = '/DeviceGray'
+        elif self.mode == 'CMYK':
+            color_space = '/DeviceCMYK'
+        else:
+            LOGGER.warning('Unknown image mode: %s', self.mode)
+            color_space = '/DeviceRGB'
+
+        extra = pydyf.Dictionary({
+            'Type': '/XObject',
+            'Subtype': '/Image',
+            'Width': width,
+            'Height': height,
+            'ColorSpace': color_space,
+            'BitsPerComponent': 8,
+            'Interpolate': interpolate,
+        })
+
+        if self.format == 'JPEG':
+            extra['Filter'] = '/DCTDecode'
+            return pydyf.Stream([self.image_data], extra)
+
+        extra['Filter'] = '/FlateDecode'
+        extra['DecodeParms'] = pydyf.Dictionary({
+            # Predictor 15 specifies that we're providing PNG data,
+            # ostensibly using an "optimum predictor", but doesn't actually
+            # matter as long as the predictor value is 10+ according to the
+            # spec. (Other PNG predictor values assert that we're using
+            # specific predictors that we don't want to commit to, but
+            # "optimum" can vary.)
+            'Predictor': 15,
+            'Columns': width,
+        })
+        if self.mode in ('RGB', 'RGBA'):
+            # Defaults to 1.
+            extra['DecodeParms']['Colors'] = 3
+        if self.mode in ('RGBA', 'LA'):
+            # Remove alpha channel from image
+            pillow_image = Image.open(io.BytesIO(self.image_data.data))
+            alpha = pillow_image.getchannel('A')
+            pillow_image = pillow_image.convert(self.mode[:-1])
+            png_data = self._get_png_data(pillow_image)
+            # Save alpha channel as mask
+            alpha_data = self._get_png_data(alpha)
+            stream = self.cache_image_data(alpha_data, alpha=True)
+            extra['SMask'] = pydyf.Stream([stream], extra={
+                'Filter': '/FlateDecode',
+                'Type': '/XObject',
+                'Subtype': '/Image',
+                'DecodeParms': pydyf.Dictionary({
+                    'Predictor': 15,
+                    'Columns': width,
+                }),
+                'Width': width,
+                'Height': height,
+                'ColorSpace': '/DeviceGray',
+                'BitsPerComponent': 8,
+                'Interpolate': interpolate,
+            })
+        else:
+            png_data = self._get_png_data(
+                Image.open(io.BytesIO(self.image_data.data)))
+
+        return pydyf.Stream([self.cache_image_data(png_data)], extra)
+
+    @staticmethod
+    def _get_png_data(pillow_image):
+        image_file = BytesIO()
+        pillow_image.save(image_file, format='PNG')
+
+        # Read the PNG header, then discard it because we know it's a PNG. If
+        # this weren't just output from Pillow, we should actually check it.
+        image_file.seek(8)
+
+        png_data = []
+        raw_chunk_length = image_file.read(4)
+        # PNG files consist of a series of chunks.
+        while raw_chunk_length:
+            # Each chunk begins with its data length (four bytes, may be zero),
+            # then its type (four ASCII characters), then the data, then four
+            # bytes of a CRC.
+            chunk_length, = struct.unpack('!I', raw_chunk_length)
+            chunk_type = image_file.read(4)
+            if chunk_type == b'IDAT':
+                png_data.append(image_file.read(chunk_length))
+            else:
+                image_file.seek(chunk_length, io.SEEK_CUR)
+            # We aren't checking the CRC, we assume this is a valid PNG.
+            image_file.seek(4, io.SEEK_CUR)
+            raw_chunk_length = image_file.read(4)
+
+        return b''.join(png_data)
+
+
+class LazyImage(pydyf.Object):
+    def __init__(self, cache, key, data):
+        super().__init__()
+        self._key = key
+        self._cache = cache
+        cache[key] = data
+
+    @property
+    def data(self):
+        return self._cache[self._key]
+
+
+class LazyLocalImage(pydyf.Object):
+    def __init__(self, filename):
+        super().__init__()
+        self._filename = filename
+
+    @property
+    def data(self):
+        return Path(self._filename).read_bytes()
+
 
 class SVGImage:
     def __init__(self, tree, base_url, url_fetcher, context):
@@ -93,58 +266,88 @@ class SVGImage:
             self._url_fetcher, self._context)
 
 
-def get_image_from_uri(cache, url_fetcher, optimize_size, url,
-                       forced_mime_type=None, context=None):
+def get_image_from_uri(cache, url_fetcher, options, url, forced_mime_type=None,
+                       context=None, orientation='from-image'):
     """Get an Image instance from an image URI."""
     if url in cache:
         return cache[url]
 
     try:
         with fetch(url_fetcher, url) as result:
+            parsed_url = urlparse(result.get('redirected_url'))
+            if parsed_url.scheme == 'file':
+                filename = url2pathname(parsed_url.path)
+            else:
+                filename = None
             if 'string' in result:
                 string = result['string']
             else:
                 string = result['file_obj'].read()
             mime_type = forced_mime_type or result['mime_type']
 
-            image = None
-            svg_exceptions = []
-            # Try to rely on given mimetype for SVG
-            if mime_type == 'image/svg+xml':
+        image = None
+        svg_exceptions = []
+        # Try to rely on given mimetype for SVG
+        if mime_type == 'image/svg+xml':
+            try:
+                tree = ElementTree.fromstring(string)
+                image = SVGImage(tree, url, url_fetcher, context)
+            except Exception as svg_exception:
+                svg_exceptions.append(svg_exception)
+        # Try pillow for raster images, or for failing SVG
+        if image is None:
+            try:
+                pillow_image = Image.open(BytesIO(string))
+            except Exception as raster_exception:
+                if mime_type == 'image/svg+xml':
+                    # Tried SVGImage then Pillow for a SVG, abort
+                    raise ImageLoadingError.from_exception(svg_exceptions[0])
                 try:
+                    # Last chance, try SVG
                     tree = ElementTree.fromstring(string)
                     image = SVGImage(tree, url, url_fetcher, context)
-                except Exception as svg_exception:
-                    svg_exceptions.append(svg_exception)
-            # Try pillow for raster images, or for failing SVG
-            if image is None:
-                try:
-                    pillow_image = Image.open(BytesIO(string))
-                except Exception as raster_exception:
-                    if mime_type == 'image/svg+xml':
-                        # Tried SVGImage then Pillow for a SVG, abort
-                        raise ImageLoadingError.from_exception(
-                            svg_exceptions[0])
-                    try:
-                        # Last chance, try SVG
-                        tree = ElementTree.fromstring(string)
-                        image = SVGImage(tree, url, url_fetcher, context)
-                    except Exception:
-                        # Tried Pillow then SVGImage for a raster, abort
-                        raise ImageLoadingError.from_exception(
-                            raster_exception)
-                else:
-                    # Store image id to enable cache in Stream.add_image
-                    image_id = hash(url)
-                    image = RasterImage(pillow_image, image_id, optimize_size)
+                except Exception:
+                    # Tried Pillow then SVGImage for a raster, abort
+                    raise ImageLoadingError.from_exception(raster_exception)
+            else:
+                # Store image id to enable cache in Stream.add_image
+                image_id = md5(url.encode()).hexdigest()
+                image = RasterImage(
+                    pillow_image, image_id, string, filename, cache,
+                    orientation, options)
 
     except (URLFetchingError, ImageLoadingError) as exception:
         LOGGER.error('Failed to load image at %r: %s', url, exception)
         image = None
+
     cache[url] = image
     return image
 
 
+def rotate_pillow_image(pillow_image, orientation):
+    """Return a copy of a Pillow image with modified orientation.
+
+    If orientation is not changed, return the same image.
+
+    """
+    image_format = pillow_image.format
+    if orientation == 'from-image':
+        if 'exif' in pillow_image.info:
+            pillow_image = ImageOps.exif_transpose(pillow_image)
+    elif orientation != 'none':
+        angle, flip = orientation
+        if angle > 0:
+            rotation = getattr(Image.Transpose, f'ROTATE_{angle}')
+            pillow_image = pillow_image.transpose(rotation)
+        if flip:
+            pillow_image = pillow_image.transpose(
+                Image.Transpose.FLIP_LEFT_RIGHT)
+
+    # Keep image format as it is discarded by transposition
+    pillow_image.format = image_format
+    return pillow_image
+
+
 def process_color_stops(vector_length, positions):
     """Give color stops positions on the gradient vector.
 
@@ -154,7 +357,7 @@ def process_color_stops(vector_length, positions):
     ``positions`` is a list of ``None``, or ``Dimension`` in px or %. 0 is the
     starting point, 1 the ending point.
 
-    See http://dev.w3.org/csswg/css-images-3/#color-stop-syntax.
+    See https://drafts.csswg.org/css-images-3/#color-stop-syntax.
 
     Return processed color stops, as a list of floats in px.
 
@@ -211,7 +414,7 @@ def normalize_stop_positions(positions):
 
 def gradient_average_color(colors, positions):
     """
-    http://dev.w3.org/csswg/css-images-3/#gradient-average-color
+    https://drafts.csswg.org/css-images-3/#gradient-average-color
     """
     nb_stops = len(positions)
     assert nb_stops > 1
@@ -284,68 +487,32 @@ class Gradient:
             if 0 not in (a0, a1) and (a0, a1) != (1, 1):
                 color_couples[i][2] = a0 / a1
 
-        shading = stream.add_shading()
-        shading['ShadingType'] = 2 if type_ == 'linear' else 3
-        shading['ColorSpace'] = '/DeviceRGB'
-        shading['Domain'] = pydyf.Array([positions[0], positions[-1]])
-        shading['Coords'] = pydyf.Array(points)
-        shading['Function'] = pydyf.Dictionary({
-            'FunctionType': 3,
-            'Domain': pydyf.Array([positions[0], positions[-1]]),
-            'Encode': pydyf.Array((len(colors) - 1) * [0, 1]),
-            'Bounds': pydyf.Array(positions[1:-1]),
-            'Functions': pydyf.Array([
-                pydyf.Dictionary({
-                    'FunctionType': 2,
-                    'Domain': pydyf.Array([0, 1]),
-                    'C0': pydyf.Array(c0),
-                    'C1': pydyf.Array(c1),
-                    'N': n,
-                }) for c0, c1, n in color_couples
-            ]),
-        })
-        if not self.repeating:
-            shading['Extend'] = pydyf.Array([b'true', b'true'])
+        shading_type = 2 if type_ == 'linear' else 3
+        domain = (positions[0], positions[-1])
+        extend = not self.repeating
+        encode = (len(colors) - 1) * (0, 1)
+        bounds = positions[1:-1]
+        sub_functions = (
+            stream.create_interpolation_function((0, 1), c0, c1, n)
+            for c0, c1, n in color_couples)
+        function = stream.create_stitching_function(
+            domain, encode, bounds, sub_functions)
+        shading = stream.add_shading(
+            shading_type, 'RGB', domain, points, extend, function)
         stream.transform(d=scale_y)
 
         if any(alpha != 1 for alpha in alphas):
-            alpha_stream = stream.add_group(
-                [0, 0, concrete_width, concrete_height])
-            alpha_state = pydyf.Dictionary({
-                'Type': '/ExtGState',
-                'SMask': pydyf.Dictionary({
-                    'Type': '/Mask',
-                    'S': '/Luminosity',
-                    'G': alpha_stream,
-                }),
-                'ca': 1,
-                'AIS': 'false',
-            })
-            stream.set_state(alpha_state)
+            alpha_stream = stream.set_alpha_state(
+                0, 0, concrete_width, concrete_height)
 
-            alpha_shading = alpha_stream.add_shading()
-            alpha_shading['ShadingType'] = 2 if type_ == 'linear' else 3
-            alpha_shading['ColorSpace'] = '/DeviceGray'
-            alpha_shading['Domain'] = pydyf.Array(
-                [positions[0], positions[-1]])
-            alpha_shading['Coords'] = pydyf.Array(points)
-            alpha_shading['Function'] = pydyf.Dictionary({
-                'FunctionType': 3,
-                'Domain': pydyf.Array([positions[0], positions[-1]]),
-                'Encode': pydyf.Array((len(colors) - 1) * [0, 1]),
-                'Bounds': pydyf.Array(positions[1:-1]),
-                'Functions': pydyf.Array([
-                    pydyf.Dictionary({
-                        'FunctionType': 2,
-                        'Domain': pydyf.Array([0, 1]),
-                        'C0': pydyf.Array([c0]),
-                        'C1': pydyf.Array([c1]),
-                        'N': 1,
-                    }) for c0, c1 in alpha_couples
-                ]),
-            })
-            if not self.repeating:
-                alpha_shading['Extend'] = pydyf.Array([b'true', b'true'])
+            shading_type = 2 if type_ == 'linear' else 3
+            sub_functions = (
+                stream.create_interpolation_function((0, 1), (c0,), (c1,), 1)
+                for c0, c1 in alpha_couples)
+            function = stream.create_stitching_function(
+                domain, encode, bounds, sub_functions)
+            alpha_shading = alpha_stream.add_shading(
+                shading_type, 'Gray', domain, points, extend, function)
             alpha_stream.transform(d=scale_y)
             alpha_stream.stream = [f'/{alpha_shading.id} sh']
 
