@@ -1,16 +1,9 @@
-"""
-    weasyprint.svg.defs
-    -------------------
-
-    Parse and draw definitions: gradients, patterns, masks, uses…
-
-"""
+"""Parse and draw definitions: gradients, patterns, masks, uses…"""
 
 from itertools import cycle
 from math import ceil, hypot
 
-import pydyf
-
+from ..matrix import Matrix
 from .bounding_box import is_valid_bounding_box
 from .utils import color, parse_url, size, transform
 
@@ -26,7 +19,13 @@ def use(svg, node, font_size):
             del node.attrib[attribute]
 
     parsed_url = parse_url(node.get_href(svg.url))
-    if parsed_url.fragment and parsed_url[:3] == parse_url(svg.url)[:3]:
+    svg_url = parse_url(svg.url)
+    if svg_url.scheme == 'data':
+        svg_url = parse_url('')
+    same_origin = (
+        parsed_url[:3] == ('', '', '') or
+        parsed_url[:3] == svg_url[:3])
+    if parsed_url.fragment and same_origin:
         if parsed_url.fragment in svg.use_cache:
             tree = svg.use_cache[parsed_url.fragment].copy()
         else:
@@ -49,7 +48,7 @@ def use(svg, node, font_size):
 
     if tree.tag in ('svg', 'symbol'):
         # Explicitely specified
-        # http://www.w3.org/TR/SVG11/struct.html#UseElement
+        # https://www.w3.org/TR/SVG11/struct.html#UseElement
         tree._etree_node.tag = 'svg'
         if 'width' in node.attrib and 'height' in node.attrib:
             tree.attrib['width'] = node.attrib['width']
@@ -61,10 +60,8 @@ def use(svg, node, font_size):
             if key not in tree.attrib:
                 tree.attrib[key] = value
 
-    svg.stream.push_state()
+    node.override_iter(iter((tree,)))
     svg.stream.transform(e=x, f=y)
-    svg.draw_node(tree, font_size)
-    svg.stream.pop_state()
 
 
 def draw_gradient_or_pattern(svg, node, name, font_size, opacity, stroke):
@@ -80,8 +77,6 @@ def draw_gradient_or_pattern(svg, node, name, font_size, opacity, stroke):
 def draw_gradient(svg, node, gradient, font_size, opacity, stroke):
     """Draw given gradient node."""
     # TODO: merge with Gradient.draw
-    from ..document import Matrix
-
     positions = []
     colors = []
     for child in gradient:
@@ -107,12 +102,13 @@ def draw_gradient(svg, node, gradient, font_size, opacity, stroke):
     bounding_box = svg.calculate_bounding_box(node, font_size, stroke)
     if not is_valid_bounding_box(bounding_box):
         return False
-    x, y = bounding_box[0], bounding_box[1]
-    matrix = Matrix(e=x, f=y)
     if gradient.get('gradientUnits') == 'userSpaceOnUse':
         width, height = svg.inner_width, svg.inner_height
+        matrix = Matrix()
     else:
-        width, height = bounding_box[2], bounding_box[3]
+        width, height = 1, 1
+        e, f, a, d = bounding_box
+        matrix = Matrix(a=a, d=d, e=e, f=f)
 
     spread = gradient.get('spreadMethod', 'pad')
     if spread in ('repeat', 'reflect'):
@@ -136,59 +132,36 @@ def draw_gradient(svg, node, gradient, font_size, opacity, stroke):
             positions.append(positions[-1] + 1)
             colors.append(colors[-1])
 
+    if 'gradientTransform' in gradient.attrib:
+        transform_matrix = transform(
+            gradient.get('gradientTransform'), font_size,
+            svg.normalized_diagonal)
+        matrix = transform_matrix @ matrix
+
     if gradient.tag == 'linearGradient':
         shading_type = 2
         x1, y1 = (
-            size(gradient.get('x1', 0), font_size, 1),
-            size(gradient.get('y1', 0), font_size, 1))
+            size(gradient.get('x1', 0), font_size, width),
+            size(gradient.get('y1', 0), font_size, height))
         x2, y2 = (
-            size(gradient.get('x2', '100%'), font_size, 1),
-            size(gradient.get('y2', 0), font_size, 1))
-        if gradient.get('gradientUnits') == 'userSpaceOnUse':
-            x1 -= x
-            y1 -= y
-            x2 -= x
-            y2 -= y
-        else:
-            length = min(width, height)
-            x1 *= length
-            y1 *= length
-            x2 *= length
-            y2 *= length
-            a = (width / height) if height < width else 1
-            d = (height / width) if height > width else 1
-            matrix = Matrix(a=a, d=d) @ matrix
+            size(gradient.get('x2', '100%'), font_size, width),
+            size(gradient.get('y2', 0), font_size, height))
         positions, colors, coords = spread_linear_gradient(
-            spread, positions, colors, x1, y1, x2, y2)
+            spread, positions, colors, x1, y1, x2, y2, bounding_box, matrix)
     else:
         assert gradient.tag == 'radialGradient'
         shading_type = 3
         cx, cy = (
-            size(gradient.get('cx', '50%'), font_size, 1),
-            size(gradient.get('cy', '50%'), font_size, 1))
-        r = size(gradient.get('r', '50%'), font_size, 1)
+            size(gradient.get('cx', '50%'), font_size, width),
+            size(gradient.get('cy', '50%'), font_size, height))
+        r = size(gradient.get('r', '50%'), font_size, hypot(width, height))
         fx, fy = (
             size(gradient.get('fx', cx), font_size, width),
             size(gradient.get('fy', cy), font_size, height))
-        fr = size(gradient.get('fr', 0), font_size, 1)
-        if gradient.get('gradientUnits') == 'userSpaceOnUse':
-            cx -= x
-            cy -= y
-            fx -= x
-            fy -= y
-        else:
-            length = min(width, height)
-            cx *= length
-            cy *= length
-            r *= length
-            fx *= length
-            fy *= length
-            fr *= length
-            a = (width / height) if height < width else 1
-            d = (height / width) if height > width else 1
-            matrix = Matrix(a=a, d=d) @ matrix
+        fr = size(gradient.get('fr', 0), font_size, hypot(width, height))
         positions, colors, coords = spread_radial_gradient(
-            spread, positions, colors, fx, fy, fr, cx, cy, r, width, height)
+            spread, positions, colors, fx, fy, fr, cx, cy, r, width, height,
+            matrix)
 
     alphas = [color[3] for color in colors]
     alpha_couples = [
@@ -209,87 +182,53 @@ def draw_gradient(svg, node, gradient, font_size, opacity, stroke):
         if 0 not in (a0, a1) and (a0, a1) != (1, 1):
             color_couples[i][2] = a0 / a1
 
+    bx1, by1 = 0, 0
     if 'gradientTransform' in gradient.attrib:
-        transform_matrix = transform(
-            gradient.get('gradientTransform'), font_size,
-            svg.normalized_diagonal)
-        matrix = transform_matrix @ matrix
+        bx1, by1 = transform_matrix.invert.transform_point(bx1, by1)
+        bx2, by2 = transform_matrix.invert.transform_point(width, height)
+        width, height = bx2 - bx1, by2 - by1
 
-    matrix = matrix @ svg.stream.ctm
-    pattern = svg.stream.add_pattern(width, height, width, height, matrix)
-    group = pattern.add_group([0, 0, width, height])
+    pattern = svg.stream.add_pattern(
+        bx1, by1, width, height, width, height, matrix @ svg.stream.ctm)
+    group = pattern.add_group(bx1, by1, width, height)
 
-    shading = group.add_shading()
-    shading['ShadingType'] = shading_type
-    shading['ColorSpace'] = '/DeviceRGB'
-    shading['Domain'] = pydyf.Array([positions[0], positions[-1]])
-    shading['Coords'] = pydyf.Array(coords)
-    shading['Function'] = pydyf.Dictionary({
-        'FunctionType': 3,
-        'Domain': pydyf.Array([positions[0], positions[-1]]),
-        'Encode': pydyf.Array((len(colors) - 1) * [0, 1]),
-        'Bounds': pydyf.Array(positions[1:-1]),
-        'Functions': pydyf.Array([
-            pydyf.Dictionary({
-                'FunctionType': 2,
-                'Domain': pydyf.Array([positions[0], positions[-1]]),
-                'C0': pydyf.Array(c0),
-                'C1': pydyf.Array(c1),
-                'N': n,
-            }) for c0, c1, n in color_couples
-        ]),
-    })
-    if spread not in ('repeat', 'reflect'):
-        shading['Extend'] = pydyf.Array([b'true', b'true'])
+    domain = (positions[0], positions[-1])
+    extend = spread not in ('repeat', 'reflect')
+    encode = (len(colors) - 1) * (0, 1)
+    bounds = positions[1:-1]
+    sub_functions = (
+        group.create_interpolation_function(domain, c0, c1, n)
+        for c0, c1, n in color_couples)
+    function = group.create_stitching_function(
+        domain, encode, bounds, sub_functions)
+    shading = group.add_shading(
+        shading_type, 'RGB', domain, coords, extend, function)
 
     if any(alpha != 1 for alpha in alphas):
-        alpha_stream = group.add_group(
-            [0, 0, svg.concrete_width, svg.concrete_height])
-        state = pydyf.Dictionary({
-            'Type': '/ExtGState',
-            'SMask': pydyf.Dictionary({
-                'Type': '/Mask',
-                'S': '/Luminosity',
-                'G': alpha_stream,
-            }),
-            'ca': 1,
-            'AIS': 'false',
-        })
-        group.set_state(state)
-
-        alpha_shading = alpha_stream.add_shading()
-        alpha_shading['ShadingType'] = shading_type
-        alpha_shading['ColorSpace'] = '/DeviceGray'
-        alpha_shading['Domain'] = pydyf.Array(
-            [positions[0], positions[-1]])
-        alpha_shading['Coords'] = pydyf.Array(coords)
-        alpha_shading['Function'] = pydyf.Dictionary({
-            'FunctionType': 3,
-            'Domain': pydyf.Array([positions[0], positions[-1]]),
-            'Encode': pydyf.Array((len(colors) - 1) * [0, 1]),
-            'Bounds': pydyf.Array(positions[1:-1]),
-            'Functions': pydyf.Array([
-                pydyf.Dictionary({
-                    'FunctionType': 2,
-                    'Domain': pydyf.Array([0, 1]),
-                    'C0': pydyf.Array([c0]),
-                    'C1': pydyf.Array([c1]),
-                    'N': 1,
-                }) for c0, c1 in alpha_couples
-            ]),
-        })
-        if spread not in ('repeat', 'reflect'):
-            alpha_shading['Extend'] = pydyf.Array([b'true', b'true'])
+        alpha_stream = group.set_alpha_state(bx1, by1, width, height)
+        domain = (positions[0], positions[-1])
+        extend = spread not in ('repeat', 'reflect')
+        encode = (len(colors) - 1) * (0, 1)
+        bounds = positions[1:-1]
+        sub_functions = (
+            group.create_interpolation_function((0, 1), [c0], [c1], 1)
+            for c0, c1 in alpha_couples)
+        function = group.create_stitching_function(
+            domain, encode, bounds, sub_functions)
+        alpha_shading = alpha_stream.add_shading(
+            shading_type, 'Gray', domain, coords, extend, function)
         alpha_stream.stream = [f'/{alpha_shading.id} sh']
 
     group.shading(shading.id)
+    pattern.set_alpha(1)
     pattern.draw_x_object(group.id)
     svg.stream.color_space('Pattern', stroke=stroke)
     svg.stream.set_color_special(pattern.id, stroke=stroke)
     return True
 
 
-def spread_linear_gradient(spread, positions, colors, x1, y1, x2, y2):
+def spread_linear_gradient(spread, positions, colors, x1, y1, x2, y2,
+                           bounding_box, matrix):
     """Repeat linear gradient."""
     # TODO: merge with LinearGradient.layout
     from ..images import gradient_average_color, normalize_stop_positions
@@ -304,7 +243,6 @@ def spread_linear_gradient(spread, positions, colors, x1, y1, x2, y2):
 
         # Define defined gradient length and steps between positions
         stop_length = last - first
-        assert stop_length > 0
         position_steps = [
             positions[i + 1] - positions[i]
             for i in range(len(positions) - 1)]
@@ -324,15 +262,33 @@ def spread_linear_gradient(spread, positions, colors, x1, y1, x2, y2):
                 [0] + position_steps + [0] + position_steps[::-1])
             previous_colors = cycle(colors + colors[::-1])
 
+        # Normalize bounding box
+        bx1, by1, bw, bh = bounding_box
+        bx1, bx2 = (bx1, bx1 + bw) if bw > 0 else (bx1 + bw, bx1)
+        by1, by2 = (by1, by1 + bh) if bh > 0 else (by1 + bh, by1)
+
+        # Transform gradient vector coordinates
+        tx1, ty1 = matrix.transform_point(x1, y1)
+        tx2, ty2 = matrix.transform_point(x2, y2)
+
+        # Find the extremities of the repeating vector, by projecting the
+        # bounding box corners on the gradient vector
+        xb, yb = tx1, ty1
+        xv, yv = tx2 - tx1, ty2 - ty1
+        xa1, xa2 = (bx1, bx2) if tx1 < tx2 else (bx2, bx1)
+        ya1, ya2 = (by1, by2) if ty1 < ty2 else (by2, by1)
+        min_vector = ((xa1 - xb) * xv + (ya1 - yb) * yv) / hypot(xv, yv) ** 2
+        max_vector = ((xa2 - xb) * xv + (ya2 - yb) * yv) / hypot(xv, yv) ** 2
+
         # Add colors after last step
-        while last < hypot(x2 - x1, y2 - y1):
+        while last < max_vector:
             step = next(next_steps)
             colors.append(next(next_colors))
             positions.append(positions[-1] + step)
             last += step * stop_length
 
-        # Add colors before last step
-        while first > 0:
+        # Add colors before first step
+        while first > min_vector:
             step = next(previous_steps)
             colors.insert(0, next(previous_colors))
             positions.insert(0, positions[0] - step)
@@ -345,7 +301,7 @@ def spread_linear_gradient(spread, positions, colors, x1, y1, x2, y2):
 
 
 def spread_radial_gradient(spread, positions, colors, fx, fy, fr, cx, cy, r,
-                           width, height):
+                           width, height, matrix):
     """Repeat radial gradient."""
     # TODO: merge with RadialGradient._repeat
     from ..images import gradient_average_color, normalize_stop_positions
@@ -357,15 +313,13 @@ def spread_radial_gradient(spread, positions, colors, fx, fy, fr, cx, cy, r,
         # Keep original lists and values, they’re useful
         original_colors = colors.copy()
         original_positions = positions.copy()
-        gradient_length = r - fr
 
         # Get the maximum distance between the center and the corners, to find
         # how many times we have to repeat the colors outside
-        max_distance = max(
-            hypot(width - fx, height - fy),
-            hypot(width - fx, -fy),
-            hypot(-fx, height - fy),
-            hypot(-fx, -fy))
+        tw, th = matrix.invert.transform_point(width, height)
+        max_distance = hypot(
+            max(abs(fx), abs(tw - fx)), max(abs(fy), abs(th - fy)))
+        gradient_length = r - fr
         repeat_after = ceil((max_distance - r) / gradient_length)
         if repeat_after > 0:
             # Repeat colors and extrapolate positions
@@ -376,7 +330,7 @@ def spread_radial_gradient(spread, positions, colors, fx, fy, fr, cx, cy, r,
                 assert spread == 'reflect'
                 colors = []
                 for i in range(repeat):
-                    colors += original_colors[::1 if i % 2 else -1]
+                    colors += original_colors[::-1 if i % 2 else 1]
             positions = [
                 i + position for i in range(repeat) for position in positions]
             r += gradient_length * repeat_after
@@ -401,7 +355,8 @@ def spread_radial_gradient(spread, positions, colors, fx, fy, fr, cx, cy, r,
             else:
                 assert spread == 'reflect'
                 for i in range(full_repeat):
-                    colors += original_colors[::1 if i % 2 else -1]
+                    colors += original_colors[
+                        ::-1 if (i + repeat_after) % 2 else 1]
             positions = [
                 i - full_repeat + position for i in range(full_repeat)
                 for position in original_positions] + positions
@@ -420,6 +375,8 @@ def spread_radial_gradient(spread, positions, colors, fx, fy, fr, cx, cy, r,
         assert 0 < partial_repeat < 1
         reverse = original_positions[::-1]
         ratio = 1 - partial_repeat
+        if spread == 'reflect':
+            original_colors = original_colors[::-1]
         for i, position in enumerate(reverse, start=1):
             if position == ratio:
                 # The center is a color of the gradient, truncate original
@@ -429,8 +386,7 @@ def spread_radial_gradient(spread, positions, colors, fx, fy, fr, cx, cy, r,
                     position - full_repeat - 1
                     for position in original_positions[-i:]]
                 positions = new_positions + positions
-                coords = (fx, fy, fr, cx, cy, r)
-                return positions, colors, coords
+                break
             if position < ratio:
                 # The center is between two colors of the gradient,
                 # define the center color as the average of these two
@@ -448,6 +404,7 @@ def spread_radial_gradient(spread, positions, colors, fx, fy, fr, cx, cy, r,
                     in original_positions[-(i - 1):]]
                 positions = (
                     [ratio - 1 - full_repeat] + new_positions + positions)
+                break
 
     coords = (fx, fy, fr, cx, cy, r)
     return positions, colors, coords
@@ -455,7 +412,6 @@ def spread_radial_gradient(spread, positions, colors, fx, fy, fr, cx, cy, r,
 
 def draw_pattern(svg, node, pattern, font_size, opacity, stroke):
     """Draw given gradient node."""
-    from ..document import Matrix
     from . import Pattern
 
     pattern._etree_node.tag = 'svg'
@@ -499,10 +455,11 @@ def draw_pattern(svg, node, pattern, font_size, opacity, stroke):
 
     matrix = matrix @ svg.stream.ctm
     stream_pattern = svg.stream.add_pattern(
-        pattern_width, pattern_height, pattern_width, pattern_height, matrix)
+        0, 0, pattern_width, pattern_height, pattern_width, pattern_height,
+        matrix)
     stream_pattern.set_alpha(opacity)
 
-    group = stream_pattern.add_group([0, 0, pattern_width, pattern_height])
+    group = stream_pattern.add_group(0, 0, pattern_width, pattern_height)
     Pattern(pattern, svg).draw(
         group, pattern_width, pattern_height, svg.base_url,
         svg.url_fetcher, svg.context)
@@ -531,11 +488,7 @@ def apply_filters(svg, node, filter_node, font_size):
         elif child.tag == 'feBlend':
             mode = child.get('mode', 'normal')
             mode = mode.replace('-', ' ').title().replace(' ', '')
-            blend_mode = pydyf.Dictionary({
-                'Type': '/ExtGState',
-                'BM': f'/{mode}',
-            })
-            svg.stream.set_state(blend_mode)
+            svg.stream.set_blend_mode(mode)
 
 
 def paint_mask(svg, node, mask, font_size):
@@ -563,21 +516,8 @@ def paint_mask(svg, node, mask, font_size):
         x, y = 0, 0
         width, height = width_ref, height_ref
 
-    alpha_stream = svg.stream.add_group([x, y, width, height])
-    state = pydyf.Dictionary({
-        'Type': '/ExtGState',
-        'SMask': pydyf.Dictionary({
-            'Type': '/Mask',
-            'S': '/Luminance',
-            'G': alpha_stream,
-        }),
-        'ca': 1,
-        'AIS': 'false',
-    })
-    svg.stream.set_state(state)
-
     svg_stream = svg.stream
-    svg.stream = alpha_stream
+    svg.stream = svg.stream.set_alpha_state(x, y, width, height)
     svg.draw_node(mask, font_size)
     svg.stream = svg_stream
 
